[lld] r286939 - [ELF] - Allow symbols of STT_NOTYPE to be associated with SHT_SYMTAB sections.

George Rimar via llvm-commits llvm-commits at lists.llvm.org
Mon Nov 14 23:56:28 PST 2016


Author: grimar
Date: Tue Nov 15 01:56:28 2016
New Revision: 286939

URL: http://llvm.org/viewvc/llvm-project?rev=286939&view=rev
Log:
[ELF] - Allow symbols of STT_NOTYPE to be associated with SHT_SYMTAB sections.

Found this when tried to link lang/ccl FreeBSD port.
Issue is very close to D23201.
This is the reason of lang/ccl port link fail.

GNU assembler 2.17.50 [FreeBSD] 2007-07-03 could generate broken objects,
where notype symbols are associated with symtab:
...
[ 9] .symtab           SYMTAB           0000000000000000  00003c78
     0000000000006858  0000000000000018          10   803     8
...
192: 000000000000000d     0 NOTYPE  LOCAL  DEFAULT    9 _cons_org

Patch allows to handle such objects.

Differential revision: https://reviews.llvm.org/D26613

Added:
    lld/trunk/test/ELF/invalid/symtab-symbols.test
Modified:
    lld/trunk/ELF/InputFiles.cpp

Modified: lld/trunk/ELF/InputFiles.cpp
URL: http://llvm.org/viewvc/llvm-project/lld/trunk/ELF/InputFiles.cpp?rev=286939&r1=286938&r2=286939&view=diff
==============================================================================
--- lld/trunk/test/ELF/invalid/symtab-symbols.test (added)
+++ lld/trunk/test/ELF/invalid/symtab-symbols.test Tue Nov 15 01:56:28 2016
@@ -0,0 +1,25 @@
+# RUN: yaml2obj %s -o %t
+# RUN: ld.lld -shared %t -o %tout
+
+# GNU assembler 2.17.50 [FreeBSD] 2007-07-03 could generate
+# broken objects.
+# Verify that lld can handle STT_NOTYPE symbols associated
+# with SHT_SYMTAB section. 
+
+!ELF
+FileHeader:
+  Class:           ELFCLASS64
+  Data:            ELFDATA2LSB
+  OSABI:           ELFOSABI_FREEBSD
+  Type:            ET_REL
+  Machine:         EM_X86_64
+Sections:
+  - Name:            .text
+    Type:            SHT_PROGBITS
+    Flags:           [ SHF_ALLOC, SHF_EXECINSTR ]
+    AddressAlign:    0x0000000000000010
+    Content:         "00000000"
+Symbols:
+  Local:
+    - Type:            STT_NOTYPE
+      Section:         .symtab




More information about the llvm-commits mailing list
--- lld/trunk/ELF/InputFiles.cpp (original)
+++ lld/trunk/ELF/InputFiles.cpp Tue Nov 15 01:56:28 2016
@@ -417,13 +417,14 @@ elf::ObjectFile<ELFT>::getSection(const
     fatal(getFilename(this) + ": invalid section index: " + Twine(Index));
   InputSectionBase<ELFT> *S = Sections[Index];
 
   // We found that GNU assembler 2.17.50 [FreeBSD] 2007-07-03
   // could generate broken objects. STT_SECTION symbols can be
   // associated with SHT_REL[A]/SHT_SYMTAB/SHT_STRTAB sections.
-  // In this case it is fine for section to be null here as we
-  // do not allocate sections of these types.
+  // In this case it is fine for section to be null here as we do not
+  // allocate sections of these types.
   if (!S) {
-    if (Index == 0 || Sym.getType() == STT_SECTION)
+    if (Index == 0 || Sym.getType() == STT_SECTION ||
+        Sym.getType() == STT_NOTYPE)
       return nullptr;
     fatal(getFilename(this) + ": invalid section index: " + Twine(Index));
   }
@@ -417,13 +417,14 @@ elf::ObjectFile<ELFT>::getSection(const
   InputSectionBase<ELFT> *S = Sections[Index];
 
-  // We found that GNU assembler 2.17.50 [FreeBSD] 2007-07-03
-  // could generate broken objects. STT_SECTION symbols can be
+  // We found that GNU assembler 2.17.50 [FreeBSD] 2007-07-03 could
+  // generate broken objects. STT_SECTION/STT_NOTYPE symbols can be
   // associated with SHT_REL[A]/SHT_SYMTAB/SHT_STRTAB sections.

Added: lld/trunk/test/ELF/invalid/symtab-symbols.test
URL: http://llvm.org/viewvc/llvm-project/lld/trunk/test/ELF/invalid/symtab-symbols.test?rev=286939&view=auto
==============================================================================